/*
    $Id: Daemon.java,v 1.1 1997/07/23 17:57:00 augusto Exp $

    Copyright (C) 1997  Augusto Ciuffoletti

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact address: augusto@di.unipi.it

    $Log: Daemon.java,v $
    Revision 1.1  1997/07/23 17:57:00  augusto
    Initial revision

*/
package syd;

import java.io.*;
import java.net.*;
import java.util.*;
import syd.logicalclock.*;
import syd.inputdata.*;
import syd.mpx.*;
import syd.session.*;
import syd.log.*;

/**
 * This is the main program of the syd package
 *
 * @version $Id: Daemon.java,v 1.1 1997/07/23 17:57:00 augusto Exp $
 * @author Augusto Ciuffoletti
 *
 */ 
public class Daemon {
/* the following line can be changed */
  private static final int MAXSESSIONS = 10;
  private static final String CONFIGFILE = "syd.conf";
/* changeable lines stop here... */
/* These are members that are statically associated to the class */
  private static Session session[];         /* the threads */
  private static Mpx channel;                /* the arbiter */
  private static LogicalClock logical_clock[];/* the logical clocks */
  private static InputData[] session_description = new InputData[MAXSESSIONS];
  private static int count_sessions = 0;
  //  private static boolean config_file = true;
  private static boolean verbose; 

  public static void main(String argv[]) 
         throws Exception {
    // Qui ci sono due cose da fare: sistemare l'update di verbose (ma e'
    // complicato perche' c'e' da rivedere il modo in cui sono passati
    // i parametri) e sistemare meglio le eccezioni nella acquisizione
    // delle descrizioni
    PortHandler port_handler = new PortHandler();
    StringBuffer report = new StringBuffer();
    Log logfile = new Log();
    int x,y,i;
 
    System.err.println("DA FARE: lock per evitare esecuzioni multiple\n");
    try { 
      load_descriptions(argv); // Parsing command line options
    } catch (ConfigurationLineException e) {
          throw new IOException(e.why + "\nline n. " + count_sessions + ": " + e.what);
    } catch (DataConsistencyException e) { 
          throw new IOException("Session "+count_sessions+": Data are inconsistent");
    }
    logfile.verbose ("\n\n\n\nDaemon restarted on " + new Date(SystemClock.read()) + "\n\n");
    start_all();
    port_handler.start(); // Starts the responding thread
    // Keep listening to successful synchronizations
    while ( (x = channel.mpx_select()) >= 0) { 
      report = new StringBuffer();
      report.append("Daemon wakes up on " + new Date(SystemClock.read()) + "\n");
      report.append("      Receive submission from "+ x + "\n");
      report.append(logical_clocks_report()); // records the state of the lc's
      if ((y = restart_one()) >= 0)
        report.append("Regenerating session " + y + "...\n");
      logfile.verbose(report.toString());
    }
/* the main returns when there are non more active logical clocks (failure) */
    return;
  }

  /**
   * MUTATOR: loads the session descriptions from the command line and from
   * the (optional) configuration file. Prints two message lines.
   *
   * @param <code>s</code> the array of Strings containing the command 
   * line arguments
   * @throws ConfigurationLineException if the command line or 
   * the description in the configuration file are wrong.
   * @throws DataConsistencyException if the data are not consistent
   * @throws IOException if there is an error reading from the configuration
   * file
   */
  public static void load_descriptions(String[] s)
       throws ConfigurationLineException, IOException,
	 DataConsistencyException, CommandLineException {
	 String s_d;
	 try {
	   session_description[count_sessions] = new InputData(count_sessions);
	   session_description[count_sessions].parse_command_line(s); 
	   session_description[count_sessions].check_consistency();
	   verbose = session_description[0].verbose; // andrebbe rimediata...
	   // Opening configuration file
	   BufferedReader configuration_data = 
	     new BufferedReader(
		 new InputStreamReader(
		     new FileInputStream(DefaultValues.home()+"/"+CONFIGFILE)));
	   System.out.println("Loading session descriptions from configuration file.");
	   // If configuration file exists, load alternate servers
	   while ( (s_d = configuration_data.readLine()) != null ) {
	     count_sessions++;
	     session_description[count_sessions] = new InputData(count_sessions);
	     session_description[count_sessions].parse_String(s_d);
	     session_description[count_sessions].check_consistency();
	     session_description[count_sessions].verbose = session_description[0].verbose; 
	}
	configuration_data.close();
	} catch (FileNotFoundException e) {
      System.out.println("Configuration file " + DefaultValues.home()+"/" + CONFIGFILE + " does not exist");
	}
      System.out.println("Loaded "+count_sessions+"+1 session descriptions!");
      count_sessions++;
  }
  /** 
   * MUTATOR: restarts one of the logical clock synchronization sessions.
   * Returns the index of the restarted session. The policy is fair, but
   * random.
   *
   * @return an int indicating the restarted session, or -1 if all sessions 
   * are active
   *
   * @throws InterruptedException if a synchronization error occurs
   */
   public static int restart_one() 
       throws InterruptedException {
     int y;
     if ((y = channel.next_not_alive()) >= 0) {
       logical_clock[y] = new LogicalClock(session_description[y].rho);
       session[y] = new Session(session_description[y],channel,logical_clock[y]);
       session[y].start();
       channel.mpx_accept(y);
     }
     return y;
   }
  /** 
   * MUTATOR: Starts all sessions: prints on the standard output a message
   * as the operation proceeds (one line).
   *
   * @throws InterruptedException if a synchronization error occurs
   */
  public static void start_all() 
       throws InterruptedException {
    int x;
    System.out.print("Starting sessions");
    channel = new Mpx(count_sessions);
    session = new Session[count_sessions];
    logical_clock = new LogicalClock[count_sessions];
    for (x=0; x<count_sessions; x++) {
      System.out.print(".");
      logical_clock[x] = new LogicalClock(session_description[x].rho);
      session[x] = new Session(session_description[x],channel,logical_clock[x]);
      session[x].start();
      channel.mpx_accept(x);
    }
    System.out.println(" done.");
  }
  /** 
   * ACCESSOR: tests whether this daemon was asked to be verbose
   *
   * @return a boolean that is <true>true</true> if daemon is verbose
   */
  public static boolean is_verbose() {
    return verbose;
  }    
  /** 
   * ACCESSOR: selects one of the logical clocks based on some policy (work
   * on progress...)
   * 
   * @return the encoding of a selected logical clock among those managed 
   * by this daemon if at least one clock is in contact, otherwise the
   * null pointer
   * @see syd.logicalclock.encode 
   */
  public static byte[] best_clock() {
    // selects the clock
    int x = 0;
    while ( x < count_sessions && !logical_clock[x].is_committed() ) x++;
    if (x < count_sessions)
      // returns encoded logical clock
      return logical_clock[x].toBytes();
    else
      return null;
  }    
  /**
   * ACCESSOR: builds a report of the current state of the clocks controlled
   * by this Daemon
   *
   * @return a String that represents the logicalclocks
   */
  public static String logical_clocks_report() {      
    int i;
    long o,p;
    long int0=Long.MIN_VALUE;
    long int1=Long.MAX_VALUE;
    StringBuffer report = new StringBuffer();

    try {
      for (i=0;i<count_sessions;i++) {
	if (logical_clock[i].is_committed()) {
	  o=logical_clock[i].offset();
	  p=logical_clock[i].precision();
	  report.append("   "+i+": offset="+o+"; delta="+p);
	  report.append("  ["+(o-p)+","+(o+p)+"]\n");
	  if ( o-p > int0 ) int0 = o-p;
	  if ( o+p < int1 ) int1 = o+p;
	} else
	  report.append("   "+i+": not in contact\n");
      }
      report.append("      Interval intersection is ["+int0+","+int1+"]");
      if (int0>int1) 
	report.append(" *****************  ERROR\n");
      else
	report.append("\n");
      return report.toString();
    } catch ( Exception e ) {
      return "An exception occurred building this string\n";
    }
  }
}

class PortHandler extends Thread {
  public void run()
  { 
    DatagramSocket socket = null;
    DatagramPacket packet = null;
    InetAddress address = null;
    int port;
    byte[] buffer = new byte[40];
    int i;

    try {
      socket = new DatagramSocket(1200);
      while (true) {
        packet = new DatagramPacket (buffer,40);
        socket.receive(packet);
        address = packet.getAddress();
        port = packet.getPort();
        System.out.print(".");
 	buffer = Daemon.best_clock();
	packet = new DatagramPacket(buffer,40,address,port);
	socket.send(packet);
      }
    } catch ( Exception e ) { 
      e.printStackTrace(); 
    }
  }
}

















