/*
    $Id: Log.java,v 1.2 1998/01/30 14:38:08 augusto Exp $

    Copyright (C) 1997  Augusto Ciuffoletti

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact address: augusto@di.unipi.it

    $Log: Log.java,v $
    Revision 1.2  1998/01/30 14:38:08  augusto
    Added timestamping of the log messages.

    Revision 1.1  1997/07/23 18:02:13  augusto
    Initial revision

*/
package syd.log;

import java.io.*;
import syd.logicalclock.*;
import syd.inputdata.*;
import syd.Daemon.*;

/**
 * A manager of the logging information produced by the program.
 *
 * @version $Id: Log.java,v 1.2 1998/01/30 14:38:08 augusto Exp $
 * @author Augusto Ciuffoletti
 *
 */
public class Log {
  private InputData sd; 
  private RandomAccessFile logfile;
  private String idString;

  /** 
   * CONSTRUCTOR: creates a new <code>Log</code> for a the session.
   *
   * @param i the <code>InputData</code> associated with a session.
   * @throws IOException if the file cannot be opened or written
   */
  public Log(InputData i) 
         throws IOException {
    sd=i;
    idString = "Session "+sd.identity;
    logfile = new RandomAccessFile(sd.logfile,"rw");
    logfile.skipBytes((int)logfile.length());
    this.verbose(sd.common_header() + sd.toString());
    return;
  }

  /** 
   * CONSTRUCTOR: creates a new <code>Log</code> for the daemon.
   *
   * @throws IOException if the file cannot be opened or written
   */
  public Log() 
         throws IOException {
    idString = "Daemon ==";
    logfile = new RandomAccessFile(DefaultValues.home()+"/log_d","rw");
    logfile.skipBytes((int)logfile.length());
    return;
  }

  /**
   * MUTATOR: writes in the log, and to the standard output if the
   * <code>Daemon</code> is verbose, a String
   *
   * @param s the String to be logged
   * @throws IOException if the file cannot be opened or written
   */
  public void verbose(String s) 
         throws IOException {
    if (syd.Daemon.is_verbose()) 
    {
      System.out.println("====== Begin msg from "+ idString + "===========");
      System.out.println(s);
      System.out.println("====== End msg from " + idString + " =============");
      }
    logfile.writeBytes("Timestamp=" + SystemClock.read() + "\n"); 
    logfile.writeBytes(s + "\n"); 
  }
  
  /**
   * MUTATOR: closes the <code>Log</code>. The Log MUST be closed after use!
   *
   * @throws IOException if the file cannot be closed
   *
   */
  public void close() 
         throws IOException {
    logfile.close();
  }
}
