/*
    $Id: NTPTime.java,v 1.1 1997/07/23 18:37:27 augusto Exp $

    Copyright (C) 1997  Augusto Ciuffoletti

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact address: augusto@di.unipi.it

    $Log: NTPTime.java,v $
    Revision 1.1  1997/07/23 18:37:27  augusto
    Initial revision

*/
package syd.remoteclock;

/**
 * This class supports the NTP encoding of time information
 *
 * @version $Id: NTPTime.java,v 1.1 1997/07/23 18:37:27 augusto Exp $
 * @author Augusto Ciuffoletti
 *
 */
public class NTPTime {
  private long integer_part;
  private long fraction_part;

  /**
   * CONSTRUCTOR: builds a new <code>NTPTime</code> instance given a long 
   * representing a time in millisecs according with Java APIs.
   * @param par a long integer representing a time (in millisecs).
   */
  public NTPTime (long par) {
  long temp;
  integer_part = par/1000;
  temp = ((par % 1000)* 4294967296L );
  fraction_part = temp / 1000;
  if (temp % 1000 > 500) fraction_part++;
  }
  /**
   * CONSTRUCTOR: builds a new <code>NTPTime</code> instance given its 
   * encoding in the NTP packet as an array of bytes.
   * @param par a byte array containing the NTP encoding of the time.
   */
  public NTPTime (byte[] par) {
    int y;
    int i;
    for (i=0;i<4;i++) {
      y=(int) par[i];
      if (y<0) y += 256;
      integer_part = (integer_part * 256) + y;
    }
    integer_part -=  2208988800L;
    for (i=4;i<8;i++) {
      y=(int) par[i];
      if (y<0) y += 256;
      fraction_part = (fraction_part * 256) + y;
    }
  }
  /**
   * ACCESSOR: returns a long integer that represents this object in
   * according with Java APIs
   * @returns a long integer representing millisecs
   */
  public long toLong() {
    double temp;
    temp = ((double)fraction_part) / 4294967296L * 1000;
    return (integer_part * 1000 ) + Math.round(temp); 
  }
  /** ACCESSOR: returns an array of bytes that represents this 
   * <code>NTPTime</code> according with the NTP encoding
   * @return an array of bytes representing this object
   */
  public byte[] toBytes() {
    int i;
    long ip,fp;
    byte data[];

    data = new byte[8];
    ip = integer_part + 2208988800L;
    for (i=3;i>=0;i--) {
      data[i] = (byte) (ip & 255);
      ip /= 256;
    }
    fp = fraction_part;
    for (i=7;i>3;i--) {
      data[i] = (byte) (fp & 255);
      fp /= 256;
    }
    return data;
  }
}




